﻿using HarmonyLib;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using WebSocketSharp;

namespace Whitebrim.LeavesAsCompost
{
    public class LeavesAsCompost : Mod
    {
        private const int THATCH_FUEL_VALUE = 5;
        private const int PLANK_FUEL_VALUE = 10;
        private const int FLOWER_FUEL_VALUE = 10;
        private const int FEATHER_FUEL_VALUE = 5;
        private List<SO_FuelValue> fuelValues;
        private List<Item_Base> itemBases;
        public static List<(string, int)> items = new List<(string, int)> { 
            ("Thatch", THATCH_FUEL_VALUE), 
            ("Plank", PLANK_FUEL_VALUE),
            ("Flower_Black", FLOWER_FUEL_VALUE),
            ("Flower_Blue", FLOWER_FUEL_VALUE),
            ("Flower_Red", FLOWER_FUEL_VALUE),
            ("Flower_White", FLOWER_FUEL_VALUE),
            ("Flower_Yellow", FLOWER_FUEL_VALUE),
            ("Feather", FEATHER_FUEL_VALUE),
        };
        Harmony harmonyInstance;

        public void Awake()
        {
            harmonyInstance = new Harmony("com.whitebrim.leavesascompost");
            harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
            CreateSO();
            if (RAPI.IsCurrentSceneGame()) 
            {
                AddFuelType();
                PatchAllCompostersManually();
            }
            Debug.Log("LeavesAsCompost has been loaded!");
        }

        private void CreateSO()
        {
            fuelValues = new List<SO_FuelValue>();
            itemBases = new List<Item_Base>();
            for (int i = 0; i < items.Count; i++)
            {
                var fuel = ScriptableObject.CreateInstance<SO_FuelValue>();
                fuel.itemType = ItemManager.GetItemByName(items[i].Item1);
                itemBases.Add(fuel.itemType);
                fuel.fuelValueOfType = items[i].Item2;
                fuel.fuelFiltrationType = FuelValue_FiltrationType.FV_COMPOSTCOMPOSTTANK;
                fuel.name = $"FuelValue_{items[i].Item1}_CompostTank";
                fuelValues.Add(fuel);
            }
        }

        public override void WorldEvent_WorldLoaded()
        {
            AddFuelType();
        }

        private void AddFuelType()
        {
            var liquidFuelManager = FindObjectOfType<LiquidFuelManager>();
            var fuelValueDictionary = Traverse.Create(typeof(LiquidFuelManager)).Field("fuelValueDictionary").GetValue<Dictionary<FuelValue_FiltrationType, List<SO_FuelValue>>>();
            foreach (var item in fuelValues)
            {
                fuelValueDictionary[FuelValue_FiltrationType.FV_COMPOSTCOMPOSTTANK].Add(item);
            }
            Traverse.Create(typeof(LiquidFuelManager)).Field("fuelValueDictionary").SetValue(fuelValueDictionary);
            // Before 1.0
            //Traverse.Create(liquidFuelManager).Fields("fuelValueDictionary").GetValue<Dictionary<FuelValue_FiltrationType, List<SO_FuelValue>>>();
            //if (Traverse.Create(liquidFuelManager).Field("initialized").GetValue<bool>())
            //{
            //    LiquidFuelManager.fuelTypesStatic = LiquidFuelManager.fuelTypesStatic.AddRangeToArray(fuelValues.ToArray());
            //}
            //liquidFuelManager.fuelTypes = liquidFuelManager.fuelTypes.AddRangeToArray(fuelValues.ToArray());
        }

        private void PatchAllCompostersManually()
        {
            var composters = FindObjectsOfType<Placeable_BiofuelExtractor>();
            if (composters != null)
            {
                //Debug.Log(composters.Length);
                foreach (var composter in composters)
                {
                    List<Item_Base> list = Traverse.Create(composter.processingTank).Field("acceptableTypes").GetValue() as List<Item_Base>;
                    list.AddRange(itemBases);
                }
            }
        }

        public void OnModUnload()
        {
            {
                var liquidFuelManager = FindObjectOfType<LiquidFuelManager>();
                var fuelValueDictionary = Traverse.Create(typeof(LiquidFuelManager)).Field("fuelValueDictionary").GetValue<Dictionary<FuelValue_FiltrationType, List<SO_FuelValue>>>();
                foreach (var item in fuelValues)
                {
                    fuelValueDictionary[FuelValue_FiltrationType.FV_COMPOSTCOMPOSTTANK].Remove(item);
                }
                Traverse.Create(typeof(LiquidFuelManager)).Field("fuelValueDictionary").SetValue(fuelValueDictionary);
            }
            var composters = FindObjectsOfType<Placeable_BiofuelExtractor>();
            if (composters != null)
            {
                foreach (var composter in composters)
                {
                    List<Item_Base> list = Traverse.Create(composter.processingTank).Field("acceptableTypes").GetValue() as List<Item_Base>;
                    for (int i = 0; i < itemBases.Count; i++)
                    {
                        list.Remove(itemBases[i]);
                    }
                }
            }

            harmonyInstance.UnpatchAll("com.whitebrim.leavesascompost");
            Debug.Log("LeavesAsCompost has been unloaded!");
            Destroy(gameObject);
        }
    }

    class Placeable_BiofuelExtractorPatch
    {
        [HarmonyPatch(typeof(Placeable_BiofuelExtractor), "OnBlockPlaced")]
        class OnBlockPlaced
        {
            private static void Prefix(Placeable_BiofuelExtractor __instance, bool ___hasBeenPlaced)
            {
                if (!___hasBeenPlaced)
                {
                    List<Item_Base> list = Traverse.Create(__instance.processingTank).Field("acceptableTypes").GetValue() as List<Item_Base>;
                    foreach (var item in LeavesAsCompost.items)
                    {
                        list.Add(ItemManager.GetItemByName(item.Item1));
                    }
                }
            }
        }
    }
}